-----------------------------------------------------------------------------------------------
-- Client Lua Script for AuctionMaxIlevelFix
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- AuctionMaxIlevelFix Module Definition
-----------------------------------------------------------------------------------------------
local AuctionMaxIlevelFix = {} 
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function AuctionMaxIlevelFix:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function AuctionMaxIlevelFix:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- AuctionMaxIlevelFix OnLoad
-----------------------------------------------------------------------------------------------
function AuctionMaxIlevelFix:OnLoad()
	--Check if Carbine addon is a new version than when this fix was implemented

	addonAH = Apollo.GetAddon("MarketplaceAuction")	

	addonAH.HelperCheckValidValues = self.fixHelperCheckValidValues
	addonAH.GetBuyFilterOptions = self.fixGetBuyFilterOptions
	addonAH.OnHelperFilterLevelEditBox = self.fixOnHelperFilterLevelEditBox
	
	local oldInitialize = addonAH.Initialize
	addonAH.Initialize = function(luaCaller)
		oldInitialize(luaCaller)
		self:SetMaxLevel()
		return oldInitialize
	end
end


-----------------------------------------------------------------------------------------------
-- AuctionMaxIlevelFix Functions
-----------------------------------------------------------------------------------------------

function AuctionMaxIlevelFix:SetMaxLevel()
	addonAH = Apollo.GetAddon("MarketplaceAuction")

	if addonAH.wndMain then
		addonAH.wndMain:FindChild("FilterOptionsLevelMaxContainer"):FindChild("FilterOptionsLevelEditBox"):SetText("120")
	end
end

--Carbine's function, modified for item level
function AuctionMaxIlevelFix:fixHelperCheckValidValues(wndChanged)
	local knMaxLevel = 120
	local knMinLevel = 1
	local wndFilterOptions = self.wndMain:FindChild("BuyContainer:AdvancedOptionsContainer:FilterContainer:FilterOptionsContainer")
	local wndMinLevelFilter = wndFilterOptions:FindChild("FilterOptionsLevelMinContainer:FilterOptionsLevelEditBox")
	local wndMaxLevelFilter = wndFilterOptions:FindChild("FilterOptionsLevelMaxContainer:FilterOptionsLevelEditBox")
	local nMinLevelValue = tonumber(wndMinLevelFilter:GetText()) or knMinLevel
	local nMaxLevelValue = tonumber(wndMaxLevelFilter:GetText()) or knMaxLevel
	local bMinChanged = false
	local bMaxChanged = false

	if wndChanged == wndMinLevelFilter then
		if nMinLevelValue < knMinLevel then
			nMinLevelValue = knMinLevel
			bMinChanged = true
		elseif nMinLevelValue >= knMaxLevel then
			nMinLevelValue = knMaxLevel
			bMinChanged = true
		end
		if nMinLevelValue > nMaxLevelValue then
			nMinLevelValue = nMaxLevelValue
			bMinChanged = true
		end
	end

	if wndChanged == wndMaxLevelFilter then
		if nMaxLevelValue < knMinLevel then
			nMaxLevelValue = knMinLevel
			bMaxChanged = true
		elseif nMaxLevelValue >= knMaxLevel then
			nMaxLevelValue = knMaxLevel
			bMaxChanged = true
		end
		if nMinLevelValue > nMaxLevelValue and ((nMinLevelValue >= 10 and nMaxLevelValue >= 10)) then
			nMaxLevelValue = nMinLevelValue
			bMaxChanged = true
		end
	end

	wndFilterOptions:FindChild("FilterOptionsRefreshBtn"):Enable(nMaxLevelValue >= nMinLevelValue)

	if bMinChanged then
		wndMinLevelFilter:SetText(nMinLevelValue)
	end
	if bMaxChanged then
		wndMaxLevelFilter:SetText(nMaxLevelValue)
	end

	wndFilterOptions:FindChild("FilterOptionsLevelMinContainer:FilterOptionsLevelUpBtn"):Enable(nMinLevelValue < knMaxLevel and nMinLevelValue < nMaxLevelValue)
	wndFilterOptions:FindChild("FilterOptionsLevelMinContainer:FilterOptionsLevelDownBtn"):Enable(nMinLevelValue > knMinLevel)
	wndFilterOptions:FindChild("FilterOptionsLevelMaxContainer:FilterOptionsLevelUpBtn"):Enable(nMaxLevelValue < knMaxLevel)
	wndFilterOptions:FindChild("FilterOptionsLevelMaxContainer:FilterOptionsLevelDownBtn"):Enable(nMaxLevelValue > knMinLevel and nMaxLevelValue > nMinLevelValue)
end

--Carbine's function, modified for item level
function AuctionMaxIlevelFix:fixGetBuyFilterOptions()
	local knMaxLevel = 120
	local knMinLevel = 1
	-- NOTE: There is support for MarketplaceLib.ItemAuctionFilterData.ItemAuctionFilterPropertyMax, but we currently aren't using it
	local tOptions = {}

	-- Equippable Filtering
	if self.wndMain:FindChild("FilterContainer"):FindChild("FilterOptionsUseable"):IsChecked() then
		table.insert(tOptions, { nType = MarketplaceLib.ItemAuctionFilterData.ItemAuctionFilterEquippableBy }) -- This can receive more arguments such as Race or Class
	end

	-- Level Filtering
	local wndFilter = self.wndMain:FindChild("FilterContainer")
	local nLevelMin = tonumber(wndFilter:FindChild("FilterOptionsLevelMinContainer"):FindChild("FilterOptionsLevelEditBox"):GetText()) or knMinLevel
	local nLevelMax = tonumber(wndFilter:FindChild("FilterOptionsLevelMaxContainer"):FindChild("FilterOptionsLevelEditBox"):GetText()) or knMaxLevel
	if nLevelMin == knMinLevel then
		wndFilter:FindChild("FilterOptionsLevelMinContainer"):FindChild("FilterOptionsLevelDownBtn"):Enable(false)
		wndFilter:FindChild("FilterOptionsLevelMinContainer"):FindChild("FilterOptionsLevelEditBox"):SetText(nLevelMin)
	end
	if nLevelMax == knMaxLevel then
		wndFilter:FindChild("FilterOptionsLevelMaxContainer"):FindChild("FilterOptionsLevelUpBtn"):Enable(false)
		wndFilter:FindChild("FilterOptionsLevelMaxContainer"):FindChild("FilterOptionsLevelEditBox"):SetText(nLevelMax)
	end
	if nLevelMin ~= knMinLevel or nLevelMax ~= knMaxLevel then
		table.insert(tOptions, { nType = MarketplaceLib.ItemAuctionFilterData.ItemAuctionFilterLevel, nMin = nLevelMin, nMax = nLevelMax })
	end

	-- Max Buyout Price
	local nMaxBuyout = tonumber(wndFilter:FindChild("FilterOptionsBuyoutCash"):GetAmount()) or 0
	if nMaxBuyout > 0 then
		table.insert(tOptions, { nType = MarketplaceLib.ItemAuctionFilterData.ItemAuctionFilterBuyoutMax, monMax = wndFilter:FindChild("FilterOptionsBuyoutCash"):GetCurrency() })
	end
	--wndFilter:FindChild("FilterOptionsBuyoutCashHelpText"):Show(nMaxBuyout == 0)

	-- Rarity
	local eRarityMin = wndFilter:FindChild("FilterOptionsRarityMinContainer"):GetData()
	local eRarityMax = wndFilter:FindChild("FilterOptionsRarityMaxContainer"):GetData()
	if eRarityMin and eRarityMax then
		table.insert(tOptions, { nType = MarketplaceLib.ItemAuctionFilterData.ItemAuctionFilterQuality, nMin = eRarityMin, nMax = eRarityMax })
	end

	-- Runes or Stats
	-- The min values can be increased and it should work, our default UI will just use 1 though
	for idx = 1, 3 do
		local tStatBtnData = wndFilter:FindChild("FilterOptionsStatBtn"..idx):GetData() -- Data can be nil, which is valid
		local nStatType = tStatBtnData and tStatBtnData[2] or nil
		local eStatProperty = tStatBtnData and tStatBtnData[3] or nil
		if eStatProperty and nStatType == MarketplaceLib.ItemAuctionFilterData.ItemAuctionFilterPropertyMin then
			table.insert(tOptions, { nType = nStatType, eProperty = eStatProperty, fMin = 1 })
		elseif eStatProperty and nStatType == MarketplaceLib.ItemAuctionFilterData.ItemAuctionFilterRuneSlot then
			table.insert(tOptions, { nType = nStatType, eMicrochipType = eStatProperty, nMin = 1 })
		end
	end

	return tOptions
end

--Carbine's function, modified for item level
function AuctionMaxIlevelFix:fixOnHelperFilterLevelEditBox(wndParent, bAdd)
	local knMaxLevel = 120
	local knMinLevel = 1
	local nOldValue = tonumber(wndParent:FindChild("FilterOptionsLevelEditBox"):GetText()) or 1
	local nNewValue = math.min(knMaxLevel, math.max(knMinLevel, bAdd and nOldValue + 1 or nOldValue - 1))

	wndParent:SetFocus()
	wndParent:FindChild("FilterOptionsLevelEditBox"):SetText(nNewValue)

	local wndEditBox = wndParent:FindChild("FilterOptionsLevelEditBox")
	self:HelperCheckValidValues(wndEditBox)

	self.wndMain:FindChild("FilterContainer"):FindChild("FilterClearBtn"):SetData(true) -- GOTCHA: Not wndParent
end

-----------------------------------------------------------------------------------------------
-- AuctionMaxIlevelFix Instance
-----------------------------------------------------------------------------------------------
local AuctionMaxIlevelFixInst = AuctionMaxIlevelFix:new()
AuctionMaxIlevelFixInst:Init()
